"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var index_1 = require("./index");
var maxError = 'It must be at most 4 characters long.';
function max5(val) {
    if (val.length > 4)
        return maxError;
}
var singleWordError = 'It must be single word.';
function isSingleWord(val) {
    if (val.split(' ').length > 1)
        return singleWordError;
}
describe('decoder', function () {
    it('works without any option', function () {
        var defaultErrorMsg = 'Not a string';
        var str = index_1.decoder({
            typeGuard: function (val) { return typeof val === 'string'; },
            getDefaultErrorMsg: function () { return defaultErrorMsg; },
            defaultParser: index_1.identity,
        });
        var nameSchema = str();
        expect(nameSchema('foo')).toEqual('foo');
        expect(nameSchema.bind(nameSchema, 1)).toThrow(defaultErrorMsg);
    });
    it('works with DecoderFullOption', function () {
        var defaultErrorMsg = 'Not a string';
        var customErrorMsg = 'It should be a string.';
        var customValidationErrorMsg = "must not contain '-'.";
        var str = index_1.decoder({
            typeGuard: function (val) { return typeof val === 'string'; },
            getDefaultErrorMsg: function () { return defaultErrorMsg; },
            defaultParser: index_1.identity,
        });
        var alphaNumSchema = str({
            parse: function (val) {
                if (typeof val === 'string' || typeof val === 'number')
                    return "" + val;
                return val;
            },
            validate: function (val, _a) {
                var _b = _a === void 0 ? { key: undefined } : _a, key = _b.key;
                if (val.includes('-'))
                    return key + " " + customValidationErrorMsg;
            },
            errorMsg: customErrorMsg,
        });
        expect(alphaNumSchema('valid string')).toEqual('valid string');
        // it parses
        expect(alphaNumSchema(1)).toEqual('1');
        // it respects custom error msg
        expect(alphaNumSchema.bind(alphaNumSchema, true)).toThrow(customErrorMsg);
        // it respects custom validator
        expect(alphaNumSchema.bind(alphaNumSchema, 'hello-there', { key: 'akey' })).toThrow(customValidationErrorMsg);
        // it respects getErrorMsg
        var schema = str({
            getErrorMsg: function (val, _a) {
                var _b = _a === void 0 ? { key: undefined } : _a, key = _b.key;
                return key + " must not be " + val;
            },
        });
        expect(schema.bind(schema, 1, { key: 'akey' })).toThrow('akey must not be 1');
        // it accepts an array of validators
        var schemaB = str({ validate: [isSingleWord, max5] });
        expect(schemaB.bind(schemaB, 'kathmandu')).toThrow(maxError);
        expect(schemaB.bind(schemaB, 'new road')).toThrow(singleWordError);
        expect(schemaB('abc')).toEqual('abc');
    });
    it('works with DecoderValidator', function () {
        var defaultErrorMsg = 'Not a string';
        var str = index_1.decoder({
            typeGuard: function (val) { return typeof val === 'string'; },
            getDefaultErrorMsg: function () { return defaultErrorMsg; },
            defaultParser: index_1.identity,
        });
        var fooSchema = str(function (val) {
            if (val !== 'foo')
                return "Its not 'foo'.";
        });
        expect(fooSchema.bind(fooSchema, 'bar')).toThrow("Its not 'foo'.");
    });
    it('works with CustomErrorMsg', function () {
        var defaultErrorMsg = 'Not a string';
        var str = index_1.decoder({
            typeGuard: function (val) { return typeof val === 'string'; },
            getDefaultErrorMsg: function () { return defaultErrorMsg; },
            defaultParser: index_1.identity,
        });
        var customErrorMsg = 'A custom error msg.';
        var fooSchema = str(customErrorMsg);
        expect(fooSchema.bind(fooSchema, 1)).toThrow(customErrorMsg);
    });
    it('supports array of validators', function () {
        var defaultErrorMsg = 'Not a string';
        var str = index_1.decoder({
            typeGuard: function (val) { return typeof val === 'string'; },
            getDefaultErrorMsg: function () { return defaultErrorMsg; },
            defaultParser: index_1.identity,
        });
        var fooSchema = str([isSingleWord, max5]);
        expect(fooSchema.bind(fooSchema, 'kathmandu')).toThrow(maxError);
        expect(fooSchema.bind(fooSchema, 'new road')).toThrow(singleWordError);
        expect(fooSchema('abc')).toEqual('abc');
    });
});
describe('string', function () {
    it('throws an error is a value is not a string', function () {
        function run() {
            return index_1.string()(1);
        }
        expect(run).toThrow('Not a string');
    });
    it('returns the value if it is a string', function () {
        var val = 'a string';
        expect(index_1.string()(val)).toEqual(val);
    });
});
describe('number', function () {
    it('throws an error is a value is not a string', function () {
        function run() {
            return index_1.number()('a string');
        }
        expect(run).toThrow('Not a number');
    });
    it('returns the value if it is a string', function () {
        var val = 1;
        expect(index_1.number()(val)).toEqual(val);
    });
});
describe('boolean', function () {
    it('throws an error if a value is not a boolean', function () {
        var boolSchema = index_1.boolean();
        expect(boolSchema.bind(boolSchema, 1)).toThrow('Not a boolean');
    });
    it('returns value if it is a boolean', function () {
        var boolSchema = index_1.boolean();
        expect(boolSchema(false)).toEqual(false);
        expect(boolSchema(true)).toEqual(true);
    });
});
describe('object', function () {
    it('returns a value if its valid', function () {
        var userSchema = index_1.object({ name: index_1.string(), address: index_1.string() });
        var val = { name: 'foo', address: 'bar' };
        expect(userSchema(val)).toEqual(val);
    });
    it('throws the errors if a value is invalid', function () {
        var userSchema = index_1.object({ name: index_1.string(), address: index_1.string() });
        var val = {};
        function run() {
            return userSchema(val);
        }
        expect(run).toThrow('{"name":"Not a string","address":"Not a string"}');
    });
    it('validates non objects', function () {
        var userSchema = index_1.object({ name: index_1.string(), address: index_1.string() });
        expect(userSchema.bind(userSchema, undefined)).toThrow('{"name":"Not a string","address":"Not a string"}');
        expect(userSchema.bind(userSchema, 1)).toThrow('{"name":"Not a string","address":"Not a string"}');
    });
    it('respects custom validator', function () {
        var expectedError = { confirm: 'It must match "password".' };
        var customValidator = function (val) {
            var password = val.password, confirm = val.confirm;
            if (password !== confirm) {
                return expectedError;
            }
        };
        var signupSchema = index_1.object({
            username: index_1.string(),
            password: index_1.string(),
            confirm: index_1.string(),
        }, customValidator);
        var rawData = {
            username: 'ausername',
            password: 'apassword',
            confirm: 'bpassword',
        };
        expect(signupSchema.bind(signupSchema, rawData)).toThrow(JSON.stringify(expectedError));
    });
    it('throws if unknown fields are passed', function () {
        var loginSchema = index_1.object({
            username: index_1.string(),
            password: index_1.string(),
        });
        var data = {
            username: 'a username',
            password: 'a password',
            unknownField1: 'unknown field 1',
            unknownField2: 'unknown field 2',
        };
        var expectedError = '{"unknownField1":"Unknown field.","unknownField2":"Unknown field."}';
        expect(loginSchema.bind(loginSchema, data)).toThrow(expectedError);
    });
    it('respects "unknownFieldErrorMsg" option', function () {
        var loginSchema = index_1.object({
            username: index_1.string(),
            password: index_1.string(),
        }, { unknownFieldErrorMsg: 'This field is not allowed.' });
        var data = {
            username: 'a username',
            password: 'a password',
            unknownField1: 'unknown field 1',
            unknownField2: 'unknown field 2',
        };
        var expectedError = '{"unknownField1":"This field is not allowed.","unknownField2":"This field is not allowed."}';
        expect(loginSchema.bind(loginSchema, data)).toThrow(expectedError);
    });
    it('does not throw if strict is false', function () {
        var loginSchema = index_1.object({
            username: index_1.string(),
            password: index_1.string(),
        }, { strict: false });
        var data = {
            username: 'a username',
            password: 'a password',
            unknownField1: 'unknown field 1',
            unknownField2: 'unknown field 2',
        };
        expect(loginSchema(data)).toEqual(expect.objectContaining({
            username: data.username,
            password: data.password,
        }));
    });
});
describe('array', function () {
    it('returns value if valid', function () {
        var namesSchema = index_1.array(index_1.string());
        var value = ['foo', 'bar'];
        expect(namesSchema(value)).toEqual(value);
    });
    it('throws error if a value is invalid', function () {
        var namesSchema = index_1.array(index_1.string());
        var value = [1, undefined];
        var expectedError = [
            { _index: 0, error: 'Not a string' },
            { _index: 1, error: 'Not a string' },
        ];
        expect(namesSchema.bind(namesSchema, value)).toThrow(JSON.stringify(expectedError));
    });
    it('validates non array value', function () {
        var namesSchema = index_1.array(index_1.string());
        var expectedError = [{ _index: 0, error: 'Not a string' }];
        expect(namesSchema.bind(namesSchema, undefined)).toThrow(JSON.stringify(expectedError));
    });
    it('validates empty array', function () {
        var namesSchema = index_1.array(index_1.string());
        var expectedError = [{ _index: 0, error: 'Not a string' }];
        expect(namesSchema.bind(namesSchema, [])).toThrow(JSON.stringify(expectedError));
    });
    it('validates empty array', function () {
        var expectedError = 'Only 2 items are allowed';
        function allowOnly2(values) {
            if (values.length > 2)
                return expectedError;
        }
        var namesSchema = index_1.array(index_1.string(), allowOnly2);
        expect(namesSchema.bind(namesSchema, ['a', 'b', 'c', 'd'])).toThrow(JSON.stringify(expectedError));
    });
});
describe('optional', function () {
    it('returns undefined if the given value is undefined', function () {
        expect(index_1.optional(index_1.string())(undefined)).toEqual(undefined);
    });
    it('returns a value if the given value is not a undefined', function () {
        var val = 'a string';
        expect(index_1.optional(index_1.string())(val)).toEqual(val);
    });
});
describe('partial', function () {
    it('makes all the fields optional', function () {
        var userSchema = index_1.partial({
            username: index_1.string(),
            password: index_1.string(),
        });
        expect(userSchema({})).toEqual({});
    });
    it('works with toggle', function () {
        var userSchema = index_1.partial({
            username: index_1.string(),
            password: index_1.toggle(index_1.string()),
        });
        expect(userSchema.bind(userSchema, {})).toThrow('');
    });
    it('makes all the fields optional', function () {
        var expectedError = {
            displayError: '"username" and "password" cannot be same.',
        };
        function customValidator(_a) {
            var username = _a.username, password = _a.password;
            if (username === password)
                return expectedError;
        }
        var userSchema = index_1.partial({
            username: index_1.string(),
            password: index_1.string(),
        }, customValidator);
        var data = { username: 'a', password: 'a' };
        expect(userSchema.bind(userSchema, data)).toThrow(JSON.stringify(expectedError));
    });
});
describe('kitchen sink', function () {
    it('works', function () {
        var userSchema = index_1.array(index_1.object({
            name: index_1.string(),
            mobile: index_1.toggle(index_1.number()),
            address: index_1.partial({
                postcode: index_1.number(),
                street1: index_1.string(),
                street2: index_1.toggle(index_1.string()),
            }),
        }));
        var value1 = [
            {
                name: 'foo',
                mobile: 1234567890,
                address: {
                    postcode: 1,
                    street1: 'street 1',
                    street2: 'street 2',
                },
            },
        ];
        expect(userSchema(value1)).toEqual(value1);
        var value2 = [
            {
                name: 'foo',
                mobile: 1234567890,
                address: {
                    postcode: 1,
                    street1: 'street 1',
                    street2: 'street 2',
                },
            },
            {
                address: { street1: 'street 1' },
            },
        ];
        var expectedError2 = '[{"_index":1,"error":{"name":"Not a string","address":{"street2":"Not a string"}}}]';
        expect(userSchema.bind(userSchema, value2)).toThrow(expectedError2);
    });
});
